// ============================================================================
// 
// Copyright (c) 1998 The CGAL Consortium
//
// This software and related documentation is part of an INTERNAL release
// of the Computational Geometry Algorithms Library (CGAL). It is not
// intended for general use.
//
// ----------------------------------------------------------------------------
//
// release       : $CGAL_Revision: CGAL-1.0$
// release_date  : $CGAL_Date: 04/05/98 $
//
// file          : test/PmOverlay/test_homogeneous.h
// source        : test/PmOverlay/test_homogeneous.h
// revision      : $Revision:  $
// revision_date : $ $
// author(s)     : Petru Pau <Petru.Pau@risc.uni-linz.ac.at>
//
// coordinator   : RISC Linz (Sabine Stifter <stifter@cyber.at>)
//
// 
// ============================================================================

#include <CGAL/Homogeneous.h>
#include <CGAL/Pm_segment_exact_traits.h>
#include <CGAL/Pm_default_dcel.h>
#include <CGAL/Planar_map_2.h>
#include <iostream.h>

#include <CGAL/Pm_overlay_for_bops.h>

typedef double			 		Basetype;
typedef CGAL_Homogeneous<Basetype>	    	Rep_class;
typedef CGAL_Pm_segment_exact_traits<Rep_class>	Pmtraits;

typedef typename Pmtraits::Point 		Point;
typedef typename Pmtraits::X_curve		Curve;

typedef CGAL_Pm_default_dcel<Pmtraits>      	Pmdcel;
typedef CGAL_Planar_map_2<Pmdcel, Pmtraits>	Planar_map;

typedef Bops_traits<Pmtraits>			Btraits;
typedef CGAL_Pm_default_dcel<Btraits> 		Bops_dcel;
typedef CGAL_Planar_map_2<Bops_dcel, Btraits> Bops_planar_map;

int main()
{

  Planar_map pm1;
  Planar_map pm2;
  Bops_planar_map pm3;
  
  Point A[3];
  A[0] = Point(0, 0, 1);
  A[1] = Point(4, 0, 1);
  A[2] = Point(0, 4, 1);
  
  pm1.insert(Curve(A[0], A[1]));
  pm1.insert(Curve(A[1], A[2]));
  pm1.insert(Curve(A[2], A[0]));

  A[0] = Point(1, 1, 1);
  A[1] = Point(4, 2, 1);
  A[2] = Point(2, 4, 1);
  
  pm2.insert(Curve(A[0], A[1]));
  pm2.insert(Curve(A[1], A[2]));
  pm2.insert(Curve(A[2], A[0]));

  CGAL_Pm_overlay_for_bops<Pmtraits>(pm1, pm2, pm3);


  if (pm3.is_valid()){
    Bops_planar_map::Halfedge_iterator hi;
    Bops_planar_map::Vertex_iterator vi;
    Bops_planar_map::Face_iterator fi;

    cout << "Vertices" << endl;
    
    for (vi=pm3.vertices_begin(); vi!=pm3.vertices_end(); ++vi){
      cout << (*vi);
      
      switch((int)(*vi).info()){
      case 1: cout << " - Red" << endl;
	break;
      case 2: cout << " - Black" << endl;
	break;
      case 3: cout << " - Red and Black" << endl;
      }
    }

    cout << "------------------" << endl;
    cout << "Edges " << endl;
    
    for (hi=pm3.halfedges_begin(); hi!=pm3.halfedges_end(); ++hi){
      if ((*hi).source().point().x()<(*hi).target().point().x() ||
	  (*hi).source().point().x()==(*hi).target().point().x() &&
	  (*hi).source().point().y()<(*hi).target().point().y()){
	cout << (*hi).source() << "-" << (*hi).target();
	switch((int)(*hi).info().edge_color){
	case 1: cout << " - Red" << endl;
	  break;
	case 2: cout << " - Black" << endl;
	  break;
	case 3: cout << " - Red and Black" << endl;
	}
      }
    }

    cout << "------------------" << endl;
    cout << "Faces " << endl;

    Bops_planar_map::Halfedge he, he_next;

    for (fi=pm3.faces_begin(); fi!=pm3.faces_end(); ++fi){
      if (!(*fi).is_unbounded()){
	he = (*fi).halfedge_on_outer_ccb();
	he_next = he;
	
	do{
	  cout << he_next.source() << "-";
	  he_next = he_next.next_halfedge();
	} while (he!=he_next);
	
	switch((int)(*fi).info()){
	case 0: cout << " No color" << endl;
	  break;
	case 1: cout << " Red" << endl;
	  break;
	case 2: cout << " Black" << endl;
	  break;
	case 3: cout << " Red and Black" << endl;
	}
      }
      else
	cout << "Unbounded face - No color" << endl;
    }
  }
  else {
    cout << "The overlay is not a valid planar map!\n";
    return -1;
  }
  return 0;
  
}
