// ============================================================================
// 
// Copyright (c) 1998 The CGAL Consortium
//
// This software and related documentation is part of an INTERNAL release
// of the Computational Geometry Algorithms Library (CGAL). It is not
// intended for general use.
//
// ----------------------------------------------------------------------------
//
// release       : $CGAL_Revision: CGAL-1.0$
// release_date  : $CGAL_Date: 04/05/98 $
//
// file          : test/PmOverlay/test_cartesian.h
// source        : test/PmOverlay/test_cartesian.h
// revision      : $Revision:  $
// revision_date : $ $
// author(s)     : Petru Pau <Petru.Pau@risc.uni-linz.ac.at>
//
// coordinator   : RISC Linz (Sabine Stifter <stifter@cyber.at>)
//
// 
// ============================================================================

#include <CGAL/Cartesian.h>
#include <CGAL/Pm_segment_epsilon_traits.h>
#include <CGAL/Pm_default_dcel.h>
#include <CGAL/Planar_map_2.h>

#include <CGAL/Pm_overlay_for_bops.h>

#include <fstream.h>

typedef double			 		Basetype;
typedef CGAL_Cartesian<Basetype>	    	Reptype;
typedef CGAL_Pm_segment_epsilon_traits<Reptype>	Pmtraits;

typedef typename Pmtraits::Point 		Point;
typedef typename Pmtraits::X_curve		Curve;

typedef CGAL_Pm_default_dcel<Pmtraits>      	Dcel;
typedef CGAL_Planar_map_2<Dcel,Pmtraits>  	Planar_map;

typedef Bops_traits<Pmtraits>			Btraits;
typedef CGAL_Pm_default_dcel<Btraits> 		Bops_dcel;
typedef CGAL_Planar_map_2<Bops_dcel, Btraits> Bops_planar_map;

int main()
{
  char nr_filename[20];
  char in1[13] = "data/pm-1.in";
  char in2[13] = "data/pm-2.in";

  Planar_map pm1;
  Planar_map pm2;
  Bops_planar_map pm3;
  
  int nr_points;
  int nr_curves;
  int i1, i2, c;
  
  Basetype x, y;

  Point A[30];

  ifstream f_stream_in1(in1);
  f_stream_in1 >> nr_points; 
 
  for (int i=0; i<nr_points; i++){
    f_stream_in1 >> x >> y;
    A[i] = Point(x, y);
  }

  f_stream_in1 >> nr_curves;

  for (int i=0; i<nr_curves; i++){
    f_stream_in1 >> i1 >> i2 >> c;
    pm1.insert(Curve(A[i1-1], A[i2-1]));
  }

  if (!pm1.is_valid()) {
    cout << "invalid map pm1!\n";
    return -1;
  }

  ifstream f_stream_in2(in2);  
  f_stream_in2 >> nr_points; 
  
  for (int i=0; i<nr_points; i++){
    f_stream_in2 >> x >> y;
    A[i] = Point(x,y);
  }

  f_stream_in2 >> nr_curves;

  for (int i=0; i<nr_curves; i++){
    f_stream_in2 >> i1 >> i2 >> c;
    pm2.insert(Curve(A[i1-1], A[i2-1]));
  }

  cout << "Insertion finished... testing map... \n";
   
  if (!pm2.is_valid()) {
    cout << "invalid map pm2!\n";
    return -1;
  }
  CGAL_Pm_overlay_for_bops<Pmtraits>(pm1, pm2, pm3);


  if (pm3.is_valid()) return 0;
  
  
}



